/*******************************************************************************/
/*                                                                             */
/*    Copyright (c) 2007 Analog Devices, Inc.  All Rights Reserved.            */
/*    This software is proprietary to Analog Devices, Inc. and its 			   */
/*	  licensors.                                  							   */
/*                                                                             */
/*    CHANGES:  07/02/2007  1.00  - initial public release  				   */
/*																			   */
/*******************************************************************************/

/*********************************************************************

Include files

*********************************************************************/
#include <stdio.h>
#include "main.h"
#include "system.h"

//#if 0
#define WriteDestByte(val) {*dest++=val;}
//#endif
#if 0
#define WriteDestByte(val) \
{\
	*dest=val;\
	if ( *dest != *(dest-0x00400000) )\
	{\
		printf ( "At source line %d (NTSC line %d): [%x] != [%x] (%x %x)\n", __LINE__, nLineNum, (dest-0x00400000), dest, (unsigned char) *(dest-0x00400000), (unsigned char) *dest );\
		exit ( 1 );\
	}\
	dest++;\
}
#endif

void *g_pVideoBufferBaseAddress;

typedef struct
{
	unsigned int SAV;
	unsigned int EAV;
} SystemCodeType;

const SystemCodeType SystemCodeMap[4] =
{
	{ 0xFF000080, 0xFF00009D },
	{ 0xFF0000AB, 0xFF0000B6 },
	{ 0xFF0000C7, 0xFF0000DA },
	{ 0xFF0000EC, 0xFF0000F1 }
};

int IsVBlankLine ( const int nLine )
{
	/* This array contains a single bit for each line in
	   an NTSC frame. */
	   
	if (	( nLine <= 18 ) ||
			( nLine >=264 && nLine <= 281 ) ||
			( nLine == 528 ) )
		return true;
		
	return false;
}

int NtscVideoOutFrameBuffInit ( char *base_address )
{
	const int nNumNTSCVoutFrames = 1;  // changed from 2 to 1 (dkl)
	const int nNumNTSCLines = 525;
	char *dest = base_address;
	
//	SetVideoBufferBaseAddress ( base_address );
	
	int nFrameNum, nLineNum;
	
	for ( nFrameNum = 0; nFrameNum < nNumNTSCVoutFrames; ++nFrameNum )
	{
		for ( nLineNum = 1; nLineNum <= nNumNTSCLines; ++nLineNum )
		{
			int offset = 0;
			int i;
			
			if ( IsVBlankLine ( nLineNum ) )
				offset ++;
				
			if ( nLineNum > 266 || nLineNum < 3 )
				offset += 2;
				
			/* Output EAV code */
			unsigned int code = SystemCodeMap[ offset ].EAV;
			WriteDestByte ( (char) (code >> 24) & 0xff );
			WriteDestByte ( (char) (code >> 16) & 0xff );
			WriteDestByte ( (char) (code >> 8) & 0xff );
			WriteDestByte ( (char) (code) & 0xff );
			
			/* Output horizontal blanking */
			for ( i = 0; i < 67*2; ++i )
			{
				WriteDestByte ( 0x80 );
				WriteDestByte ( 0x10 );
			}
			
			/* Output SAV */
			code = SystemCodeMap[ offset ].SAV;
			WriteDestByte ( (char) (code >> 24) & 0xff );
			WriteDestByte ( (char) (code >> 16) & 0xff );
			WriteDestByte ( (char) (code >> 8) & 0xff );
			WriteDestByte ( (char) (code) & 0xff );
		
			/* Output empty horizontal data */
			for ( i = 0; i < 360*2; ++i )
			{
				WriteDestByte ( 0x80 );
				WriteDestByte ( 0x10 );
			}
		}
	}
	
	return dest - base_address;
}

void Write_To_Frame( unsigned short X, unsigned short Y, unsigned int val, void * buffer )
{
	int *pCanvas = ( int * ) buffer;
	
	/* Using 0xf10000ff as a marker for the beginning of an NTSC frame */
	if ( 0xf10000ff == *pCanvas )
	{
		unsigned int * base;
	
		/* Live video buffer */
		base = (unsigned int *) buffer;
	
		/* Effectively line-doubling in this routine */
		base += (unsigned int) 0x79BC/4;
	
		/* Convert from cartesian coords (Y get higher as we move up) to video addressing
		   (Y gets higher as we move down) */
		Y = NTSC_NUM_LINES-Y;
		base = base + (429*Y);
		base = base + X;
		*base = val;

		/* Fill in the same pixel on the second field */
		base += (unsigned int) ( 0x75CA8/4 - 0x79BC/4 );
		*base = val;
	}
	else
	{
		

		/* Some other canvas.  Assuming the first eight bytes indicate the width and
		   height of the following block (which is what adi_graph "happens" to use. :) */
		int width = *pCanvas++;
		int height = *pCanvas++;

		pCanvas += ( width * Y ) + X;
		*pCanvas = val;
	}
}

unsigned int Read_From_Frame(unsigned short X, unsigned short Y, void *buffer )
{
	int *pCanvas = ( int * ) buffer;
	
	/* Using 0xf10000ff as a marker for the beginning of an NTSC frame */
	if ( 0xf10000ff == *pCanvas )
	{
		unsigned int * base;
	
		/* Live video buffer */
		base = (unsigned int *) buffer;
	
		base += (unsigned int) 0x79BC/4;
	
		/* Convert from cartesian coords (Y get higher as we move up) to video addressing
		   (Y gets higher as we move down) */
		Y = NTSC_NUM_LINES-Y;
		base = base + (429*Y);
		base = base + X;
		return *base;
	}
	else
	{
		/* Some other canvas.  Assuming the first eight bytes indicate the width and
		   height of the following block (which is what adi_graph "happens" to use. :) */
		int width = *pCanvas++;
		int height = *pCanvas++;

		pCanvas += ( width * Y ) + X;
		return *pCanvas;
	}
}


void FillFrame (char *Frame, int Value) {

	int *OddPtr32;
	int OddLine;
	int *EvenPtr32;
	int EvenLine;
	int i;

	// fill odd and even frames
	for (OddLine = 22, EvenLine = 285; OddLine < 263; OddLine++, EvenLine++) {
		OddPtr32 = (int *)((Frame + (OddLine * 1716)) + 276);
		EvenPtr32 = (int *)((Frame + (EvenLine * 1716)) + 276);
		for (i = 0; i < 360; i++, OddPtr32++, EvenPtr32++) {
			*OddPtr32 = Value;
			*EvenPtr32 = Value;
		}
	}

	// fill odd frame
//	for (line = 22; line < 263; line++) {
//		Ptr32 = (int *)((Frame + (line * 1716)) + 276);
//		for (i = 0; i < 360; i++, Ptr32++) {
//			*Ptr32 = Value;
//		}
//	}
	
	// fill even frame
//	for (line = 285; line < 525; line++) {
//		Ptr32 = (int *)((Frame + (line * 1716)) + 276);
//		for (i = 0; i < 360; i++, Ptr32++) {
//			*Ptr32 = Value;
//		}
//	}
	
	// return
}
	
	
		
